/* SetThreadAbsolutePriority
** Allows the caller to set a thread's priority
** to any value in the real-time priority range.
**
** Author: Jeff Claar
*/
#define WIN32_LEAN_AND_MEAN
#define STRICT
#include <windows.h>
#include "priority.h"

/*
** Function typedefs and names for functions
** imported from ntdll.dll.
*/
static LPCSTR _NtSetInfoThread = "NtSetInformationThread";
static LPCSTR _RtlNtStatusToDosError = "RtlNtStatusToDosError";
typedef LONG (__stdcall *SETINFOTHREAD)(HANDLE, int, PVOID, ULONG);
typedef DWORD (__stdcall *RTLNTSTATUSTODOSERROR)(LONG);


BOOL SetThreadAbsolutePriority(
    HANDLE hThread,
    KPRIORITY priority)
{
    BOOL bSuccess = FALSE;

    /* Variables to store function pointers in ntdll.dll */
    HINSTANCE hNtDll;
    SETINFOTHREAD ntsit;
    RTLNTSTATUSTODOSERROR nterr;
    LONG status;

    /* Save old priority in case the function fails. */
    int nOldPriority = GetThreadPriority(hThread);

    if (nOldPriority == THREAD_PRIORITY_ERROR_RETURN)
    {
        /*
        ** Can't get the old priority.
        ** The thread handle might be invalid.
        */
        return FALSE;
    }

    /*
    ** First, set the thread priority to time-critical.
    ** Time-critical threads are never dynamially boosted
    ** by Windows NT.
    */
    if (SetThreadPriority(hThread, THREAD_PRIORITY_TIME_CRITICAL))
    {
        hNtDll = LoadLibraryA("ntdll.dll");
        if (hNtDll != NULL)
        {
            /*
            ** Attempt to get the address of
            ** NtSetInformationThread. (Note:
            ** This call will fail under Windows 95/98.)
            */
            ntsit = (SETINFOTHREAD)
            GetProcAddress(hNtDll, _NtSetInfoThread);
            if (ntsit != NULL)
            {
                status = ntsit(hThread,
                   2,
                   &priority,
                   sizeof(priority));

                if (status != 0)
                {
                    /*
                    ** On failure, the function returns an NT
                    ** status code. This is converted to
                    ** a Win32 error here.
                    */
                    nterr = (RTLNTSTATUSTODOSERROR)
                        GetProcAddress(hNtDll,
                        _RtlNtStatusToDosError);
                    if (nterr != NULL)
                    {
                        SetLastError(nterr(status));
                    }
                }
                else
                {
                   bSuccess = TRUE;
                }
            }
            FreeLibrary(hNtDll);
        }
    }

   /*
   ** If the function failed, reset to the
   ** original priority.
   */
   if (!bSuccess)
      SetThreadPriority(hThread, nOldPriority);

   return bSuccess;
}

